############################################################
# ARGENET (Algorithm for Reconstruction of GEne NEYwork    #
# Author: Paola Lecca, CIBIO - University of Trento, Italy #
############################################################


Args <- commandArgs();      # retrieve args
Args <- Args[3:length(Args)]
print(Args)
tic <- function(gcFirst = TRUE, type=c("elapsed", "user.self", "sys.self"))
{
   type <- match.arg(type)
   assign(".type", type, envir=baseenv())
   if(gcFirst) gc(FALSE)
   tic <- proc.time()[type]         
   assign(".tic", tic, envir=baseenv())
   invisible(tic)
}

toc <- function()
{
   type <- get(".type", envir=baseenv())
   toc <- proc.time()[type]
   tic <- get(".tic", envir=baseenv())
   print(toc - tic)
   write(toc - tic, file=Args[11])
   invisible(toc)
}



################################################################################


#################################################################################


#################################################################################

# Definition of functions


# Select genes whose RPKM level is greater 0.5 in almost 1 sample 
select_RPKM <- function(matx)
                       {
                        select_rows <- vector("list", dim(matx)[1])
                        keep_the_rows <- array(0, dim(matx)[1])
                        
                        for (i in 1:dim(matx)[1])
                            {
                             count <- 0
                             select_rows[[i]] <- matx[i,]
                             for (j in 1:dim(matx)[2])
                                 {
                                   if (select_rows[[i]][j] < 0.5)
                                      {count <- count+1}
                                 }
                             if (count==dim(matx)[2])
                                {keep_the_rows[i] <- i}
                             }
                             keep_the_rows <- keep_the_rows[ keep_the_rows != 0 ]
                             keep_the_rows 
                       }



# Flatten the correlation matrix
flattenSquareMatrix <- function(m) {
  if( (class(m) != "matrix") | (nrow(m) != ncol(m))) stop("Must be a square matrix.") 
  if(!identical(rownames(m), colnames(m))) stop("Row and column names must be equal.")
  ut <- upper.tri(m)
  data.frame(i = rownames(m)[row(m)[ut]],
             j = rownames(m)[col(m)[ut]],
             cor=t(m)[ut],
             p=m[ut])
}

# Calculate correlation and its significance
cor.prob <- function(X, dfr = nrow(X) - 2) {
   R <- cor(X, method="kendall")
   above <- row(R) < col(R)
   r2 <- R[above]^2
   Fstat <- r2 * dfr / (1 - r2)
   R[above] <- 1 - pf(Fstat, 1, dfr)
   R
        }
    

# Calculate standard error on the correlation
#cor.se <- function(X.cor.flat)
#                  {
#          Xd <- dim(X.cor.flat)[1]
#          for (i in 1:Xd)
#              {
#             c.err <- sqrt((1 - (X.cor.flat[,3])^2)/(Xd -2))
#             }
#          }
  
# Calculate the minimum number of samples for a good correlation
# (i.e. for acorrelation affected by a small standard error)

add_n_sample_min <- function(X.cor.flat)
                        {
             Xd <- dim(X.cor.flat)[1]
                 for (i in 1:Xd)
                      { 
                  # we impose that the standard error on the correlation does have to exceed the 30%
                  delta.c <- as.numeric(X.cor.flat[,3])/3
                      n.min <- 2 + (1 - (as.numeric(X.cor.flat[,3]))^2)/(delta.c)^2
              }
            X.cor.new <- cbind(X.cor.flat, n.min)
            X.cor.new
               }
                     
               
    
# bayesian variabel selection for network inference 
bayesian_vselection <- function(m, nc, fileout_name)
           {
            suppressWarnings(library(varbvs))
            #options(digits=20)
            library(MASS)

            # if(Args[11] == "TRUE") {
            #      m.corr <- as.matrix(read.table(Args[12], header=TRUE, fill=TRUE))
            #  } else { 
              mc <- cor.prob(m)
    
              m.corr <- as.matrix(flattenSquareMatrix(mc))


            #    write.matrix(m.corr, file = Args[12])
            #}
        

            if(length(m.corr) == 0){
              write(toc(), file=Args[11])
              file.create(Args[7])
              file.create(fileout_name)
              quit()
            }
              m.corr <- subset(m.corr, abs(as.numeric(m.corr[,3])) > as.numeric(Args[8]) & as.numeric(m.corr[,4]) < as.numeric(Args[9]))
            # Correlations and optimal number of samples 

            if(length(m.corr) == 0){
              write(toc(), file=Args[11])
              file.create(Args[7])
              file.create(fileout_name)
              quit()
            }     


             # m.corr.optn <- add_n_sample_min(m.corr)
             # n.samples.optimal <- max(as.numeric(m.corr.optn[,5]))
             # n.opt <- paste ("The optimal number of samples is", n.samples.optimal, sep=" ")
             # write(n.opt, file=Args[9])



            
            names_1 <- as.vector(m.corr[,1])
            names_2 <- as.vector(m.corr[,2])

            print("First loop of Bayesian inference starts")
                        results_right <- vector("list", dim(m.corr)[1])
                        m_transp <- t(m)
                        vec <- c(1:nc)
                        log_val<-log(0.0001/0.0099)
                        for (i in 1:(dim(m.corr)[1]))
                            {
                             X_right <- (as.matrix(m_transp[m.corr[i,1],vec]))
                             y_right <- as.vector(m_transp[m.corr[i,2],vec])
                             results_right[[i]] <- varbvsoptimize(X=X_right, y=y_right, 
                                               verbose=FALSE, sigma=4, sa=2, log_val)
                           #sigma=var(X_right[,1] - (y_right)),
                                                    #sa=var(c(y_right)))
                            }
                       print("First loop of Bayesian inference terminated")
                       print("Second loop of Bayesian inference starts")
                       # Probability of the right-arrows " <-- "
    
                       results_left <- vector("list", dim(m.corr)[1])

                       for (i in 1:(dim(m.corr)[1]))
                           {
                            X_left <- (as.matrix(m_transp[m.corr[i,2],vec]))
                            y_left <- as.vector(m_transp[m.corr[i,1],vec])
                            results_left[[i]] <- varbvsoptimize(X=X_left, y=y_left, 
                                              verbose=FALSE, sigma=4, sa=2, log_val)
                      #sigma=var(X_left[,1] - (y_left)), 
                                              #sa=var(c(y_left)))
                           } 
                       print("Second loop of Bayesian inference terminated")
                       
             print("Determination of directions.")
             parameter_matrix <- matrix(0, dim(m.corr)[1], 6)

                       for (i in 1:(dim(m.corr)[1]))
                           {
                            parameter_matrix[i, ] <- c(results_right[[i]]$alpha, results_left[[i]]$alpha,  
                                                       results_right[[i]]$mu,  results_left[[i]]$mu,
                                                       results_right[[i]]$lnZ, results_left[[i]]$lnZ)
                           }

                       colnames(parameter_matrix, do.NULL = FALSE)
                       colnames(parameter_matrix) <- c("a_dx", "a_sx", "mu_dx", "mu_sx", "lnZ_dx", "lnZ_sx")

                       
                       frecce <- array("-->", dim(m.corr)[1])

                       for (i in 1:(dim(m.corr)[1]))
                           {
                            if ( exp(parameter_matrix[i,5]) < exp(parameter_matrix[i,6]) )
                                 {
                                 frecce[i] <- "<--"
                                }
                           }

                       print("Determination of directions terminated.")
                       write(t(c("Gene_Symbol", "Directed_edge", "Gene_Symbol", "alpha_dx", "alpha_sx", "mu_dx", "mu_sx", "lnZ_dx", "lnZ_sx", "Correlation", "P.value")),  
                             file=fileout_name, ncolumns=3+dim(parameter_matrix)[2]+2, sep="\t")   
                             write(t(cbind(names_1, frecce, names_2, parameter_matrix, m.corr[,3], m.corr[,4])), 
                             file=fileout_name, ncolumns=3+dim(parameter_matrix)[2]+2, append=TRUE, sep="\t")

                       result <- cbind(names_1, frecce, names_2, parameter_matrix, m.corr[,3], m.corr[,4])
                       result
                       }


# Input uploading 

tic()

print("Read the input file")

obs <- (as.matrix(read.table(Args[4], sep="\t", header=T)))
filter <- (as.matrix(read.table(Args[5], sep="\t", header=T)))

obs.db <- obs[,colnames(obs) %in% colnames(filter)][,-1]

obs.db <- t(apply(obs.db, 1,as.numeric))

genes_names <- (as.matrix(read.table(Args[4], sep="\t", header=T)))[,1]

print("Input uploaded")


print("Assign names to the matrix of RPKM")
rownames(obs.db, do.NULL = FALSE)
rownames(obs.db) <- genes_names
#rownames(obs.db) <- LETTERS[1:dim(obs.db)[1]]    
            
print("Selection of signifincatly exressed genes.")            
#obs <- (obs.db[-select_RPKM(obs.db),] )
obs <- obs.db
print("Expressed genes selected.")             
print(dim(obs))

   
   
print("Bayesian inference starts")

argenet_mm <- bayesian_vselection(t(obs), dim(t(obs))[1], Args[6])


alpha <- array(0, dim(argenet_mm)[1])
mu <- array(0, dim(argenet_mm)[1])


for (i in 1:dim(argenet_mm)[1])
    {
     if (argenet_mm[i,2] =="-->")
        {
         alpha[i] <- argenet_mm[i,4]
         mu[i] <- argenet_mm[i,6]
        }
    if (argenet_mm[i,2] =="<--")
        {
         alpha[i] <- argenet_mm[i,5]
         mu[i] <- argenet_mm[i,7]
        }
}

argenet_out <- cbind(argenet_mm[,1], argenet_mm[,2], argenet_mm[,3], alpha, mu, argenet_mm[,10], argenet_mm[,11])

print("Save the outputs")
write(t(c("Gene_Symbol", "Directed_edge", "Gene_Symbol", "alpha", "mu", "Correlation", "P.value")),  
                             file=Args[7], ncolumns=7, sep="\t")  
write(t(argenet_out), file=Args[7], sep="\t", ncolumn=7, append=TRUE)

print("Finished!")

toc()
