import sys
import os

def jaccard_similarity(A_map, B_map):
	intersection=0.0

	if(((len(A_map) == 0) and (len(B_map)==0)) or len(set(A_map).union(set(B_map))) == 0):
		return 0

	for a in A_map:
		try:
			val = B_map[a][0]
			if(val * A_map[a][0]) > 0:
				intersection += 1
		except Exception: 
  			pass

	return float(intersection) / len(set(A_map).union(set(B_map)))


def jaccard_similarity_denominator(A_map, B_map):
	intersection=0.0

	if(((len(A_map) == 0) and (len(B_map)==0)) or min(len(A_map), len(B_map))==0):
		return 0

	for a in A_map:
		try:
			val = B_map[a][0]
			if(val * A_map[a][0]) > 0:
				intersection += 1
		except Exception: 
  			pass

	return float(intersection) / min(len(A_map), len(B_map))



def refined_similarity(A_map, B_map):
	intersection=0.0

	if(((len(A_map) == 0) and (len(B_map)==0)) or len(set(A_map).union(set(B_map))) == 0):
		return 0

	for a in A_map:
		try:
			val = B_map[a][0]

			if((val * A_map[a][0]) > 0):
				if(A_map[a][0] < B_map[a][0]):
					if(A_map[a][2] >= B_map[a][1]):
						intersection += 1
				else:
					if(B_map[a][2] >= A_map[a][1]):
						intersection += 1
		except Exception: 
  			pass

	return float(intersection) /  len(set(A_map).union(set(B_map)))

def refined_similarity_denominator(A_map, B_map):
	intersection=0.0

	if(((len(A_map) == 0) and (len(B_map)==0)) or min(len(A_map), len(B_map))==0):
		return 0

	for a in A_map:
		try:
			val = B_map[a][0]

			if((val * A_map[a][0]) > 0):
				if(A_map[a][0] < B_map[a][0]):
					if(A_map[a][2] >= B_map[a][1]):
						intersection += 1
				else:
					if(B_map[a][2] >= A_map[a][1]):
						intersection += 1
		except Exception: 
  			pass

	return float(intersection) /   min(len(A_map), len(B_map))


def weight_similarity(A_map, B_map):
	intersection=0.0

	if(((len(A_map) == 0) and (len(B_map)==0)) or len(set(A_map).union(set(B_map))) == 0):
		return 0

	for a in A_map:
		try:
			val = B_map[a][0]

			if((val * A_map[a][0]) > 0):		
				intersection += (1 - abs(val - A_map[a][0]))
		except Exception: 
  			pass

	return float(intersection) /   len(set(A_map).union(set(B_map)))

def weight_similarity_denominator(A_map, B_map):
	intersection=0.0

	if(((len(A_map) == 0) and (len(B_map)==0)) or min(len(A_map), len(B_map))==0):
		return 0
		
	for a in A_map:
		try:
			val = B_map[a][0]

			if((val * A_map[a][0]) > 0):		
				intersection += (1 - abs(val - A_map[a][0]))
		except Exception: 
  			pass

	return float(intersection) /   min(len(A_map), len(B_map))

if __name__ == '__main__':

	file1=sys.argv[1]
	file2=sys.argv[2]

	arc_A_map = {}
	arc_B_map = {}

	if(os.stat(file1)[6]!=0):
		with open(file1) as f:
			for row in f.readlines():
				gene_from, gene_to, weight  = row.split('\t')
				arc_A_map[str(gene_from) + "-" + str(gene_to)]=(float(weight))

	if(os.stat(file2)[6]!=0):				
		with open(file2) as f:
			for row in f.readlines():
				gene_from, gene_to, weight  = row.split('\t')
				arc_B_map[str(gene_from) + "-" + str(gene_to)]=(float(weight))
	
	print jaccard_similarity(arc_A_map, arc_B_map)
	print jaccard_similarity_denominator(arc_A_map, arc_B_map)
	print refined_similarity(arc_A_map, arc_B_map)
	print refined_similarity_denominator(arc_A_map, arc_B_map)
	print weight_similarity(arc_A_map, arc_B_map)
	print weight_similarity_denominator(arc_A_map, arc_B_map)